/*--------------------------------------------------------------
// file: vdist.c - compute vector distance matrix
//   D = vdist(A,B)
// INPUTS:
//   A - mxd matrix (m=number of points, d=number of dimensions)
//   B - nxd matrix (n=number of points, d=number of dimensions)
// OUTPUTS:
//   D - m x n x d multidimensional array of vector distances
//   the first  m x n page has the first coordinate in the subtraction
//       second m x n page has the second coordinate in the subtraction 
//       etc.
//
// Checked: 2005-11-12 (WWX)
//
//  Written by:
//  -- 
//  John L. Weatherwax                2005-10-30
//  
//  email: wax@alum.mit.edu
//  
//  Please send comments and especially bug reports to the
//  above email address.
//  
//--------------------------------------------------------------*/
#include <math.h>
#include "mex.h"

/*--------------------------------------------------------------
// function: mexFunction - Entry point from Matlab environment
// INPUTS:
//   nlhs - number of left hand side arguments (outputs)
//   plhs[] - pointer to table where created matrix pointers are
//            to be placed
//   nrhs - number of right hand side arguments (inputs)
//   prhs[] - pointer to table of input matrices
//--------------------------------------------------------------*/
void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )
{
  int M,N,D; 
  int dimensions[3],subs[3]; /* Sizes */
  int ra,rb,d,index; /* Loop variables */
  double *pA, *pB, *pC;

  if (nrhs < 2)
    mexErrMsgTxt("Not enough input arguments.");
  if (nrhs > 2)
    mexErrMsgTxt("Too many input arguments.");
  if (!mxIsDouble(prhs[0]) || !mxIsDouble(prhs[1]))
    mexErrMsgTxt("Arguments must be type double.");
  if (mxGetN(prhs[0]) != mxGetN(prhs[1]))
    mexErrMsgTxt("matrix dimensions (number of columns) must agree.");
  if (mxIsComplex(prhs[0]) || mxIsComplex(prhs[1]))
    mexWarnMsgTxt("Complex parts ignored.");

  /* vdist(MxD,NxD) => M x N x D */
  M = mxGetM(prhs[0]); /* Number of rows on input */ 
  N = mxGetM(prhs[1]); /* Number of rows in input */
  D = mxGetN(prhs[0]); /* Number of dimensions */

  /* Create output nd array */
  dimensions[0] = M; dimensions[1] = N; dimensions[2] = D;
  plhs[0] = mxCreateNumericArray(3,dimensions,mxDOUBLE_CLASS,mxREAL); 

  pA = mxGetPr(prhs[0]);
  pB = mxGetPr(prhs[1]);
  pC = (double *)mxGetData(plhs[0]);

  /* Now the distance computation */
  for( ra=0; ra<M; ra++ ){ 
    subs[0] = ra; 
    for( rb=0; rb<N; rb++ ){
      subs[1] = rb; 
      for( d=0; d<D; d++ ){
	subs[2] = d;
	index = mxCalcSingleSubscript(plhs[0],3,subs); 
        pC[index] = pA[ra+M*d] - pB[rb+N*d];
      }
    }
  }
} /* end mexFunction() */
